#!/bin/sh
#
# Copyright (C) 2016 NetFlow Logic. All rights reserved.
#
# Script for NetFlow Optimizer configuration backup

ARG0="${0}"
while [ -h "${ARG0}" ]; do
	ls=$(ls -ld "${ARG0}")
	link=$(expr "${ls}" : '.*-> \(.*\)$')
	if $(expr "${link}" : '/.*' > /dev/null); then
		ARG0="${link}"
	else
		ARG0="$(dirname ${ARG0})/${link}"
	fi
done

declare appRoot="$(readlink -f $(dirname ${ARG0}))"

declare msgConfigChangeDetected="Changed config files have been detected. They could be restored \
manually from the archive "

function checkLastOperationResult
{
	local __res=$?
	if [[ ${__res} != 0 ]] ; then
		continue
	fi
}

function backupConfigFiles
{
	if [[ -f "./.conf-info" ]];then
		local __s=$(md5sum --check .conf-info 2>/dev/null | sed -n "/FAILED/p" | sed "s/\(.*\):[[:space:]]\+FAILED/\1/")
		if [[ -n "${__s}" ]];then
			local __dt=$(date +%Y-%m-%d_%H-%M-%S)
			local __archName="conf-backup.${__dt}"
			local __dirName=""
			local __baseName=""
			local __backupName=""
			local __srcName=""
			if [[ -d "conf-backup/${__archName}/" ]];then
				rm -rf conf-backup/${__archName}/
			fi
			mkdir -p "conf-backup/${__archName}/"

			local -i __cnt=0
			for v1 in ${__s}
			do
				__srcName=$(readlink -e ${v1})
				if [[ -f "${__srcName}" ]];then
					__dirName="$(dirname ${v1})"
					__baseName="$(basename ${v1})"
					__backupName=$(stat -c %y ${v1} | sed "s/\(.*\)\..*/\1/;s/[[:space:]]/_/g;s/:/-/g")
					checkLastOperationResult
					__backupName=$(readlink -m "conf-backup/${__archName}/${__dirName}/${__baseName}.${__backupName}.backup")
					checkLastOperationResult
					mkdir -p "$(dirname ${__backupName})"
					checkLastOperationResult
					cp -rfT ${__srcName} ${__backupName}
					checkLastOperationResult
					(( __cnt++ ))
				fi
			done

			if  (( ${__cnt} > 0 ));then
				pushd "conf-backup/${__archName}/" > /dev/null
				tar -cz --owner=0 --group=0 ./* > ../${__archName}.tar.gz
				popd > /dev/null
				rm -rf conf-backup/${__archName}/
				echo -e '\E[31;40m'"${msgConfigChangeDetected}'$(readlink -e conf-backup/${__archName}.tar.gz)'."
				tput sgr0
			fi
		fi
	fi

	return 0
}

pushd ${appRoot} >> /dev/null

backupConfigFiles

popd > /dev/null
