#!/bin/bash
#
# Copyright (C) 2012 - 2016 NetFlow Logic. All rights reserved.
#
# NetFlow Optimizer startup/shutdown script
#
#==============================================================
# Options:
# start - start NetFlow Optimizer
# restart - restart NetFlow Optimizer
# reload - restart NetFlow Optimizer
# stop - stop NetFlow Optimizer
# status - get NetFlow Optimizer status
#==============================================================

MSG_INFO="The NetFlow Optimizer web interface is at "
MSG_NOT_CONFIGURED="NetFlow Optimizer is not configured, please reinstall it."
MSG_NOT_CONFIGURED_1="NetFlow Optimizer is not configured, please execute setup.sh first."
MSG_START_FAILED="NetFlow Optimizer daemon start up failed."
MSG_STOP_FAILED="NetFlow Optimizer daemon stop failed."
MSG_START_SUCCESS="NetFlow Optimizer daemon has successfully started up."
MSG_STOP_SUCCESS="NetFlow Optimizer daemon has successfully stopped."
MSG_NFI_STARTED="NetFlow Optimizer daemon is running."
MSG_NFI_STOPPED="NetFlow Optimizer daemon stopped."
MSG_NFI_UNKNOWN_STATUS="NetFlow Optimizer daemon status is unknown."
MSG_NFI_UNKNOWN_INSTANCE="Unknown instance of NetFlow Optimizer daemon is running."
MSG_NFI_EMPTY_PID_FILE="PID file exists, but process id is not found."

declare -i e_START_ERROR=64
declare -i e_STOP_ERROR=65
declare -i e_INCORRECT_ARG=70
declare -i e_NFI_NOT_CONFIGURED=101
declare -i e_UNKNOWN_ERROR=113

nodeName=""
appPath="https://localhost:8443/"
declare -i errCode=${e_UNKNOWN_ERROR}
declare -i isDaemon=1

if [[ -n "${2}" && "${2}" == "daemon" ]]; then
	isDaemon=0
fi

ARG0="${0}"
while [ -h "${ARG0}" ]; do
	ls=$(ls -ld "${ARG0}")
	link=$(expr "${ls}" : '.*-> \(.*\)$')
	if $(expr "${link}" : '/.*' > /dev/null); then
		ARG0="${link}"
	else
		ARG0="$(dirname ${ARG0})/${link}"
	fi
done

declare appRoot="$(readlink -f $(dirname ${ARG0}))"

pushd ${appRoot} >> /dev/null

JAVA_HOME="${appRoot}/java/jre8"
JEXEC="${JAVA_HOME}/bin/java"

if [[ ! -f tomcat/conf/.tomcat_keystore || ! -x "${JEXEC}" ]] ; then
	if [[ -f "${appRoot}/setup.sh" ]] ; then
		echo -e '\E[31;40m'"${MSG_NOT_CONFIGURED_1}" >&2
	else
		echo -e '\E[31;40m'"${MSG_NOT_CONFIGURED}" >&2
	fi
	tput sgr0
	errCode=${e_NFI_NOT_CONFIGURED}
	exit ${errCode}
fi

case "${1}" in
	start )
		errCode=${e_START_ERROR}

		pushd tomcat/bin >> /dev/null

		./daemon.sh --java-home ${JAVA_HOME} --service-start-wait-time 120 --tomcat-user root start
		errCode=$?

		popd >> /dev/null

		if [[ ${errCode} -eq 0 ]]; then
			echo "${MSG_START_SUCCESS}"
			if [[ isDaemon -ne 0 ]]; then
				nodeName=$(uname -n)
				appPath=${appPath/"localhost"/${nodeName}}
				echo -e '\E[32;40m'"${MSG_INFO}${appPath}"
				tput sgr0
			fi
		else
			if [[ isDaemon -ne 0 ]]; then
				echo -e '\E[31;40m'"${MSG_START_FAILED}" >&2
				tput sgr0
			fi
		fi
	;;
	stop )
		errCode=${e_STOP_ERROR}

		pushd tomcat/bin >> /dev/null

		./daemon.sh stop
		errCode=$?

		popd >> /dev/null

		if [[ ${errCode} == 0 ]]; then
			echo "${MSG_STOP_SUCCESS}"
		else
			if [[ isDaemon -ne 0 ]]; then
				echo -e '\E[31;40m'"${MSG_STOP_FAILED}" >&2
				tput sgr0
			fi
		fi
	;;
	restart | reload )
		${ARG0} stop ${2}
		${ARG0} start ${2}
		errCode=$?
	;;
	status )
		errCode=0
		cnt=0
		if [[ -f "${appRoot}/tomcat/bin/catalina.pid" ]];then
			pidNFI="$(cat ${appRoot}/tomcat/bin/catalina.pid)"
			if [[ -n "${pidNFI}" ]];then
				cnt=$(ps -p "${pidNFI}" | grep -c 'jsvc')
			else
				cnt=-1
			fi
		fi
		cnt1=$(ps aux | grep -c 'jsvc.exec.*\/flowintegrator\/tomcat')
		if [[ ${cnt} -eq 1 ]]; then
			if [[ ${cnt1} -eq 2 ]]; then
				echo "${MSG_NFI_STARTED} (PID: ${pidNFI})."
			elif [[ ${cnt1} -lt 2 ]]; then
				echo "${MSG_NFI_UNKNOWN_STATUS}"
			else
				echo "${MSG_NFI_STARTED} (PID: ${pidNFI})."
				echo "${MSG_NFI_UNKNOWN_INSTANCE}"
			fi
		elif [[ ${cnt} -eq 0 ]]; then
			echo "${MSG_NFI_STOPPED}"
			if [[ ${cnt1} -gt 0 ]]; then
				echo "${MSG_NFI_UNKNOWN_INSTANCE}"
			fi
		elif [[ ${cnt} -eq -1 ]]; then
			echo "${MSG_NFI_UNKNOWN_STATUS}"
			echo "${MSG_NFI_EMPTY_PID_FILE}"
		fi
	;;
	* )
		errCode=${e_INCORRECT_ARG}
		if [[ isDaemon -ne 0 ]]; then
			echo "Usage: ${0} {start|stop|restart|reload|status}"
		fi
esac

popd >> /dev/null

exit ${errCode}
